/** "Hello, world." program for HyperCast 2.0
*
* @author Multimedia Networks Group
* @author HyperCast Team 
* @version Hello World (Callback Version) 1.0, Nov 2001
*/


import java.util.*;
import java.io.*;
import java.net.*;
import hypercast.*;
import hypercast.events.*;

public class HelloWorld_CallBack_Notification implements I_ReceiveCallback
{ 
/******************************************************
        Data Members 
******************************************************/

     /* This is the string we will sent */ 
     String MyString = new String("Hello World");

     /* MySocket is the name of the overlay socket */
     I_OverlaySocket MySocket = null;

     /* The configuration object contains configuration  parameters */
     HyperCastConfig ConfObj = null;

/******************************************************
   The functions include:
        1. Creates an overlay socket
        2. Sends "Hello World" to all members
        3. Receives and prints "Hello World"s received 
        from other members in the overlay 
******************************************************/

public HelloWorld_CallBack_Notification ()
{
	
	/* Create an overlay configuration object */

	ConfObj = HyperCastConfig.createConfig("hypercast.xml");
	//ConfObj = HyperCastConfig.createOLConfig("hypercast.prop");

	/* Create notification handler */
	CallBack_notification_handler cnh = new CallBack_notification_handler();

	/* Finally, the overlay socket is created with the configuration object 
		"this" is the object that implements the callback 
	*/
	MySocket=ConfObj.createOverlaySocket(this, cnh);

	/* Once the overlay socket is create, it can join the overlay */ 
	MySocket.joinOverlay();

	/* Now we wait for 3 seconds. This is enough 
		time to have the overlay stabilize 
	*/

	try {
	       Thread.sleep(3000);
	} catch (InterruptedException e) {}

	/* wait for the overlay socket to be stable for sending and receiving data */
	//cnh.waitUntilStable();


	/* Print the logical address */ 
	System.out.println("This node's logical address is: " +MySocket.getLogicalAddress().toString());

	for(;;) {

	   /* Convert the string to a byte array and create a message */ 

	   byte[] MyData = MyString.getBytes(); 
	   I_OverlayMessage MyMessage = MySocket.createMessage(MyData);


	   /* The message is sent to all members of the overlay */
	   MySocket.sendToAll(MyMessage);
	   System.out.println("<HelloWorld> message sent to other members ...");

            try {
               Thread.sleep(2000);
            } catch (InterruptedException e) {}
	}
}


/*****************************************************
This the callback function of the I_ReceiveCallback interface. 
The callback is invoked each time a message arrives. 
This callback merely displays the received message 
******************************************************/


public void ReceiveCallback(I_OverlayMessage RecvdMessage)
{
     /* get the logical address of the sender */ 
     String SenderSrc = RecvdMessage.getSourceAddress().toString();

     /* Next we transfor the received message into a string */
     String sMsg=new String(RecvdMessage.getPayload(), 0, 
     RecvdMessage.getPayloadLength());

     /* ... and print the payload */ 
	 //if(!RecvdMessage.getSrcLogicalAddress().equals(MySocket.getLogicalAddress()))
	 System.out.println("Received <"+ sMsg + ">from node with logical address: " +SenderSrc +" . ");
}


public static void main(String[] args)
{

	

	/**Create a new HelloWorld_CallBack instance*/
	HelloWorld_CallBack_Notification hw = new HelloWorld_CallBack_Notification();

}

}

class CallBack_notification_handler extends NotificationHandler 
{
	/** The object used for synchronization that makes application to wait
	 * for the overlay socket to be stable before sending and receiving data.
	 */
	 Object StableGuard;

	/* Flag indicating if overlay socket is stable */
	boolean isSTABLE = false;

	CallBack_notification_handler() {
		super();

		/* create synchronization object */
		StableGuard = new Object();

	}
	
	public void waitUntilStable() {
		synchronized (StableGuard)
		{
			while (!isSTABLE) {
				try 
				{
				   System.out.println("Wait for the overlay socket being stable.");
				   StableGuard.wait();
				}
				catch (InterruptedException ie) 
				{
				}
			}
		}
	}
	
	public void handler (NOTIFICATION_EVENT obj)
	{
            if (obj instanceof NODE_ISSTABLE) {
			NODE_ISSTABLE ne = (NODE_ISSTABLE) obj;
			System.out.println("event NODE_ISSTABLE happens!");

			synchronized (StableGuard) 
			{
				isSTABLE = true;
				StableGuard.notify();
			}
		}
		else if (obj instanceof NODE_LEAVEOVERLAY) {
			NODE_LEAVEOVERLAY ne = (NODE_LEAVEOVERLAY) obj;
			System.out.println("event NODE_LEAVEGROUP happens!");
		}
		else if (obj instanceof NODE_LOGICALADDRESSCHANGED) {
			NODE_LOGICALADDRESSCHANGED ne = (NODE_LOGICALADDRESSCHANGED) obj;
			System.out.println("event NODE_LOGICALADDRESSCHANGED happens!");
		}
		else if (obj instanceof NODE_NEIGHBORHOODCHANGED) {
			NODE_NEIGHBORHOODCHANGED ne = (NODE_NEIGHBORHOODCHANGED) obj;
			System.out.println("event NODE_NEIGHBORHOODCHANGED happens!");
		}
		else
		   throw new IllegalArgumentException("unknown event happens!");
		   
	}
}
